"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _path = _interopRequireDefault(require("path"));

var _webpack = _interopRequireDefault(require("webpack"));

var _webpackDevMiddleware = _interopRequireDefault(require("webpack-dev-middleware"));

var _webpackHotMiddleware = _interopRequireDefault(require("webpack-hot-middleware"));

var _logger = _interopRequireDefault(require("@phenomic/core/lib/logger"));

var _UtilsBs = require("@phenomic/core/lib/Utils.bs.js");

var _webpackPromise = _interopRequireDefault(require("./webpack-promise.js"));

var _WebpackGetConfig = _interopRequireDefault(require("./WebpackGetConfig.js"));

var _WebpackServerConfigModifier = _interopRequireDefault(require("./WebpackServerConfigModifier.js"));

const debug = require("debug")("phenomic:plugin:bundler-webpack");

const pluginName = "@phenomic/plugin-bundler-webpack";
const log = (0, _logger.default)(pluginName);
const cacheDir = (0, _UtilsBs.findCacheDirectory)("webpack");

const bundlerWebpack = config => {
  return {
    name: pluginName,

    addDevServerMiddlewares() {
      debug("get middlewares");
      const webpackConfig = (0, _WebpackGetConfig.default)(config); // $FlowFixMe interface sucks

      const compiler = (0, _webpack.default)(webpackConfig);
      let assets = {}; // $FlowFixMe interface sucks

      compiler.hooks.done.tap(pluginName + "/dev-server-middleware", stats => {
        assets = {};
        const namedChunks = stats.compilation.namedChunks;
        namedChunks.forEach((chunk, chunkName) => {
          const files = chunk.files.filter(file => !file.endsWith(".hot-update.js"));

          if (files.length) {
            assets = (0, _objectSpread2.default)({}, assets, {
              [chunkName]: files.shift()
            });
          }
        });
      });
      return [(req, res, next) => {
        res.locals.assets = assets;
        next();
      }, (0, _webpackDevMiddleware.default)(compiler, {
        publicPath: config.baseUrl.pathname,
        logLevel: "warn",
        stats: webpackConfig.stats // logger: log, // output info even if logLevel: "warn"

      }), (0, _webpackHotMiddleware.default)(compiler, {
        reload: true,
        log
      })];
    },

    buildForPrerendering() {
      debug("build for prerendering");
      return (0, _webpackPromise.default)((0, _WebpackServerConfigModifier.default)(config, cacheDir)).then( // $FlowFixMe no I can't
      () => require(_path.default.join(cacheDir, config.bundleName)).default);
    },

    build() {
      debug("build");
      return (0, _webpackPromise.default)((0, _WebpackGetConfig.default)(config)).then(stats => stats.toJson().assetsByChunkName);
    }

  };
};

var _default = bundlerWebpack;
exports.default = _default;