"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _path = _interopRequireDefault(require("path"));

var _getClientEnvironment = _interopRequireDefault(require("@phenomic/core/lib/configuration/get-client-environment.js"));

var _webpack = _interopRequireDefault(require("webpack"));

var _miniCssExtractPlugin = _interopRequireDefault(require("mini-css-extract-plugin"));

var _UtilsBs = require("@phenomic/core/lib/Utils.bs.js");

// $FlowFixMe lazy me
const cacheDir = (0, _UtilsBs.findCacheDirectory)("webpack-babel");

module.exports = config => ({
  // https://webpack.js.org/configuration/stats/#stats
  stats: "minimal",
  mode: process.env.NODE_ENV,
  entry: {
    [config.bundleName]: [process.env.PHENOMIC_ENV !== "static" && require.resolve("webpack-hot-middleware/client"), _path.default.join(config.path, "App.js")].filter(item => item)
  },
  output: (0, _objectSpread2.default)({
    publicPath: config.baseUrl.pathname,
    path: _path.default.isAbsolute(config.outdir) ? config.outdir : _path.default.join(config.path, config.outdir)
  }, process.env.PHENOMIC_ENV !== "static" ? {
    filename: "phenomic/[name].js",
    chunkFilename: "phenomic/[name].chunk.js"
  } : {
    filename: "phenomic/[name].[chunkhash:8].js",
    chunkFilename: "phenomic/[name].[chunkhash:8].chunk.js"
  }),
  module: {
    rules: [{
      test: /\.js$/,
      exclude: /node_modules/,
      use: [{
        loader: require.resolve("babel-loader"),
        options: {
          cacheDirectory: cacheDir,
          presets: [require("@phenomic/babel-preset")]
        }
      }]
    }, {
      test: /\.css$/,
      use: [process.env.PHENOMIC_ENV !== "static" ? "style-loader" : _miniCssExtractPlugin.default.loader, require.resolve("css-loader")]
    }]
  },
  // $FlowFixMe lazy me
  plugins: [process.env.PHENOMIC_ENV === "static" && new _miniCssExtractPlugin.default({
    filename: "phenomic/[name].[chunkhash:8].css",
    chunkFilename: "phenomic/[name].[chunkhash:8].chunk.css"
  }), (() => {
    const envVars = (0, _getClientEnvironment.default)(config);
    return new _webpack.default.DefinePlugin({
      "process.env": Object.keys(envVars).reduce((env, key) => {
        env[key] = JSON.stringify(envVars[key]);
        return env;
      }, {})
    });
  })(), process.env.PHENOMIC_ENV !== "static" && new _webpack.default.HotModuleReplacementPlugin()].filter(item => item),
  resolve: {
    // react-native(-web) | react-primitives
    extensions: [".web.js", ".js", ".json"]
  },
  // eslint-disable-next-line max-len
  // https://github.com/facebookincubator/create-react-app/blob/fbdff9d722d6ce669a090138022c4d3536ae95bb/packages/react-scripts/config/webpack.config.prod.js#L279-L285
  node: {
    fs: "empty",
    net: "empty",
    tls: "empty"
  }
});